from pyrate.main import Pyrate


class BasecampPyrate(Pyrate):

    # request
    base_url = None
    default_header_content = None
    default_body_content = None
    auth_data = {'type': 'BASIC'}
    send_json = True

    # response
    response_formats = ['json']
    default_response_format = None
    validate_response = True

    connection_check = {'http_method': 'GET', 'target': 'people/me'}

    def __init__(self, auth_user, auth_pass, org_id, default_response_format=None):
        super(BasecampPyrate, self).__init__()
        self.auth_data['username'] = auth_user
        self.auth_data['password'] = auth_pass
        self.auth_data['org_id'] = org_id
        self.base_url = 'https://basecamp.com/' + org_id + '/api/v1/'
        self.default_header_content = {
            'Authorization': self.get_auth_data(),
            'User Agent': 'Pyrate'
        }

        if default_response_format:
            self.default_response_format = default_response_format
        else:
            self.default_response_format = self.response_formats[0]



from pyrate.main import Pyrate
from pyrate.utils import clean_dict, ExceptionWithCode


class ListNotFoundError(Exception):
    pass


class MailchimpPyrate(Pyrate):

    # request
    base_url = None  # see __init__
    default_header_content = None
    default_body_content = None  # see __init__
    auth_data = {'type': 'API_KEY'}
    send_json = True

    # response
    response_formats = ['JSON', 'XML', 'PHP']
    default_response_format = None
    validate_response = True

    connection_check = {'http_method': 'POST', 'target': 'helper/ping'}

    def __init__(self, apikey, default_response_format=None):
        super(MailchimpPyrate, self).__init__()
        self.base_url = 'https://' + apikey.split('-')[-1] + '.api.mailchimp.com/2.0/'
        self.default_body_content = {
            'apikey': apikey
        }

        if default_response_format:
            self.default_response_format = default_response_format
        else:
            self.default_response_format = self.response_formats[0]

    def get_auth_data(self):
        return None

    def check_response(self, response):
        if self.validate_response and not 200 <= response.status_code < 300:
            if 'code' in response.json():
                raise ExceptionWithCode(code=response.json()['code'])
            return super(MailchimpPyrate, self).check_response(response)
        return True

    # http://apidocs.mailchimp.com/api/2.0/lists/list.php
    def get_all_lists(self, filters=None, start=None, limit=None,
                      sort_field=None, sort_dir=None):

        res = self.post('lists/list', content=clean_dict(locals()))
        return res['data']

    def get_list_by_name(self, list_name):
        lists = self.get_all_lists()
        for l in lists:
            if l['name'] == list_name:
                return l

        raise ListNotFoundError

    # http://apidocs.mailchimp.com/api/2.0/lists/subscribe.php
    def subscribe_to_list(
            self, user_email, list_name=None, list_id=None, merge_vars=None,
            email_type=None, double_optin=None, update_existing=None,
            replace_interests=None, send_welcome=None):

        if not list_id:
            if not list_name:
                raise Exception('Either list_id or list_name has to be passed')
            list_id = self.get_list_by_name(list_name)['id']

        kwargs = clean_dict({
            'id': list_id, 'email': {'email': user_email},
            'merge_vars': merge_vars, 'email_type': email_type,
            'double_optin': double_optin, 'update_existing': update_existing,
            'replace_interests': replace_interests, 'send_welcome': send_welcome
        })

        return self.post('lists/subscribe', content=kwargs)
        # return self.check_response_success(res)

    # http://apidocs.mailchimp.com/api/2.0/lists/unsubscribe.php
    def unsubscribe_from_list(
            self, user_email, list_name=None, list_id=None, delete_member=None,
            send_goodbye=None, send_notify=None):

        if not list_id:
            if not list_name:
                raise Exception('Either list_id or list_name has to be passed')
            list_id = self.get_list_by_name(list_name)['id']

        kwargs = clean_dict({
            'id': list_id, 'email': {'email': user_email},
            'delete_member': delete_member, 'send_goodbye': send_goodbye,
            'send_notify': send_notify
        })

        return self.post('lists/unsubscribe', content=kwargs)
        # return self.check_response_success(res)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
from pyrate import __version__

REQUIREMENTS = (
    'requests',
    'requests-oauthlib',
    'six'
)

CLASSIFIERS = [
    'Environment :: Console',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Topic :: Communications',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
]

setup(
    name='pyrate',
    version=__version__,
    description='Pyrate is a python wrapper for restful web apis. It\'s like magic but simpler.',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author='Kim Thoenen',
    author_email='kim@smuzey.ch',
    url='https://github.com/chive/pyrate',
    packages=find_packages(),
    license='MIT',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    include_package_data=True,
    zip_safe=False,
    test_suite='pyrate.tests',
    install_requires=REQUIREMENTS,
    tests_require=(
        'httmock',
    ),
    entry_points={
        'console_scripts': [
            'pyratetools = pyrate.scripts.cliutils:main',
        ]
    },)
